//ETAPE3
/*Changements par rapport  l'tape 2 : 
 - On importe le package java.util.* car on utilise les ensembles (HashSet) et les vecteurs de taille dynamique (ArrayList) 
   dfinis dans celui-ci.
 - On a dplac les constantes relatives au type de case de la classe Constantes (qui n'existe plus) vers cette classe. On a introduit un 
   nouveau type de cellule : CELLULE_ERRONEE : cellule dont la formule est erronee. 
 - Nouvelle constante : TEXTE_CASE_ERRONEE.
 - Nouvelles variables globales : peutEtreEvaluee, formule, cellulesLiees, positionCellule
 - Il n'y a plus qu'un constructeur (qui cre une cellule vide) et celui-ci ncessite maintenant qu'on lui passe les coordonnes qu'a
   la cellule dans le tableau de la reprsentation interne (utile pour savoir quelles cellules redessiner).
 - Nouvelles mthodes : ajouteCelluleLiee, supprimeCelluleLiee, delieDesAutresCellules utilises pour savoir quelles cellules redessiner
   quand on modifie une instance de CaseRepresentationInterne.
 - Nouvelles mthodes : donnePosition, donneListeCellulesLiees, donnePositionCellulesLiees.
 - Nouvelle mthode analyseFormule : est appele si on a dtect qu'une expression est susceptible d'tre une formule (commence par un '=') 
 - Le code qui vrifie si une expression de type String est un nombre est dlocalis dans la mthode isValidNumber(String expression).
   A l'tape 2, ce code tait directement prsent dans la mthode setValue. On a fait ce choix pour allger le code de la mthode setValue,
   mais surtout parce que ce code est aussi appel dans le code analysant les formules.
 - La mthode setValue renvoit maintenant un vecteur (ventuellement vide) de Coordonnees avec les coordonnees des cellules  redessiner
 - La mthode deleteValue n'existe plus (il suffit de faire setValue("") pour avoir le mme effet)
 - Nouvelle mthode : testeDefinitionCirculaire
 - Nouvelle mthode : evalue
 - La mthode getExternalRepresentation sait maintenant valuer la formule de la case si elle est de type math et si elle en contient une.
*/
import java.util.*;    //Dfinit les classes HashSet et ArrayList

//Chaque case du tableau de la reprsentation interne sera une instance de cette classe
public class CaseRepresentationInterne {

    //Constantes
    public static final String TEXTE_CASE_ERRONEE = "####";   //S'il y a une erreur dans une formule, la mthode getExternalRepresentation renverra ce texte

    //Les diffrents types possibles pour une cellule de la reprsentation interne
    public static final int CELLULE_VIDE = 0;   //Une case vide du tableau
    public static final int CELLULE_MATH = 1;   //Une case de type math
    public static final int CELLULE_TEXTE = 2;  //Une case de type texte
    public static final int CELLULE_ERRONEE = 3;  //Une case contenant une formule invalide

    //Varibles d'instance prives
    private String value;                           //le texte de la case
    private int type;                               //le type de case (voir les constantes dfinies juste au dessus)
    private boolean peutEtreEvaluee;                //si une case est de type math et si elle contient un nombre ou une formule valide, cette variable vaudra true
    private ElementFormule formule;                 //si la case contient une formule valide, on aura une rfrence vers le sommet de l'arbre de la formule dans cette variable    
    private HashSet cellulesLiees=new HashSet();    //dans cet ensemble on gardera une rfrence vers toutes les cellules qui ont besoin de l'valuation de cette cellule pour s'valuer
    private Coordonnees positionCellule;            //chaque instance de CaseRepresentationInterne retiendra ses coordonnes dans positionCellule

 
    //CONSTRUCTEUR : Initialise une case vide. La cellule devra savoir sa position dans le tableau de la reprsentation interne, ce qu'on lui communique par (ligne, colonne)
    public CaseRepresentationInterne(int ligne, int colonne) {
	//On met  la cellule les paramtres relatifs  une case vide
	value = "";
	type = CELLULE_VIDE;
	peutEtreEvaluee = false;
	formule = null;
	positionCellule = new Coordonnees(ligne, colonne);       //On va retenir la position qu'a la cellule dans le tableau de la reprsentation interne
    }

    //Mthode publique ajouteCelluleLiee(CaseRepresentationInterne cell) : signale  this que cell a besoin de l'valuation de this pour calculer son rsultat
    //Ceci est utile quand on modifie une cellule, il faudra signaler  toutes les cellules dont le rsultat dpend de l'valuation de this de se rvaluer
    public void ajouteCelluleLiee(CaseRepresentationInterne cell) {
	cellulesLiees.add(cell);   //On ajoute cell  l'ensemble cellulesLiees si cell n'est pas dj dans l'ensemble
    }

    //Mthode publique supprimeCelluleLiee(CaseRepresentationInterne cell) : signale  this que cell n'a plus besoin de l'valuation de this pour calculer son rsultat
    public void supprimeCelluleLiee(CaseRepresentationInterne cell) {
	cellulesLiees.remove(cell);   //On enlve cell de l'ensemble cellulesLiees si cell tait dans l'ensemble
    }

    //Mthode publique donnePosition() renvoit la position de la cellule dans le tableau
    public Coordonnees donnePosition() {
	return positionCellule;
    }

    //Mthode prive delieDesAutresCellules() : signale  toutes les cases dont cette case dpendait pour son valuation que cette case ne dpend plus d'elles
    //ceci est utile quand on modifie le contenu de la cellule : on commence par signaler  toutes les cellules que cette cellule ne dpend plus d'elles
    private void delieDesAutresCellules(CaseRepresentationInterne[][] representationInterne) {
	//On va parcourir tout le tableau de la reprsentation interne et demander  chaque case de supprimer sa rfrence vers this
	for (int i=0; i < RepresentationInterne.NB_LIGNES; i++) {
	    for (int j=0; j < RepresentationInterne.NB_COLONNES; j++) {
		representationInterne[i][j].supprimeCelluleLiee(this);  //La rfrence vers this est supprime si elle existait, rien n'est fait sinon
	    }
	}
    }

    //Mthode prive HashSet donneListeCellulesLiees(HashSet parcours) : renvoit un ensemble contenant les coordonnes de toutes les cellules lies  this ainsi que les coordonnes
    //de toutes les cellules lies aux cellules lies  this ... cette mthode fonctionne donc rcursivement
    //Ici aussi pourrait survenir un problme li aux dfinitions circulaires, c'est pourquoi on retient le parcours des cellules dans l'ensemble parcours
    private HashSet donneListeCellulesLiees(HashSet parcours) throws FormuleException {
	CaseRepresentationInterne cell;      //Variable de travail
	HashSet liste = new HashSet();       //Dans liste on stockera toutes les cellules lies, directement ou non,  la cellule this

	//On commence par s'assurer qu'on n'est pas dans une dfinition circulaire
	testeDefinitionCirculaire(parcours);   //Si une dfinition circulaire est dtecte, une FormuleException est lance

	//On va examiner 1--1 les cellules lies  la case this, leur demander de donner la liste de leurs cellules lies et les ajouter  l'ensemble liste
	Iterator i = cellulesLiees.iterator();     //Un Iterator permet de lister un  un les lments d'un ensemble 
	while (i.hasNext()) {   //Tant qu'il y a des lments dans l'ensemble
	    cell = (CaseRepresentationInterne)i.next();   //on met dans cell un lment de l'ensemble, donc une des cellules lies
	    liste.add(cell);                              //on commence par ajouter cell  la liste des cellules lies
	    //puis on met dans liste les cellules liees  cell (si une cellule est dj prsente dans liste, elle ne sera pas remise une deuxime fois).
	    //On doit faire une copie du parcours car une mme cellule peut tre appele  plusieurs endroits diffrents sans pour autant faire de boucle infinie
	    //ce qui provoque la boucle, c'est une cellule qui s'appelle elle-mme
	    liste.addAll(cell.donneListeCellulesLiees(new HashSet(parcours)));   
	}
	//Ici liste contient toutes les cellules lies  cell ainsi que toutes les cellules lies  ses cellules lies
	return liste;  
    }

    //Mthode publique Coordonnees[] donnePositionCellulesLiees() : renvoit un vecteur de Coordonnees avec l'emplacemenent des cellules lies  this dans la reprsentation interne
    public Coordonnees[] donnePositionCellulesLiees() throws FormuleException {
	//On cre un vecteur de CaseRepresentationInterne contenant les rfrences vers toutes les cellules lies  this
	//A la mthode toArray on passe une rfrence vers un tableau de CaseRepresentationInterne pour qu'elle sache qu'elle devra renvoyer un tableau du mme type
	//Sinon, on aurait des ClassCastException. 
	CaseRepresentationInterne[] cells = (CaseRepresentationInterne[])donneListeCellulesLiees(new HashSet()).toArray(new CaseRepresentationInterne[0]);  //new HashSet car on commence le parcours

	Coordonnees[] positions = new Coordonnees[cells.length];  //On cre un vecteur de coordonnes qui contiendra autant de coordonnes qu'il y a de cellules lies
	for (int i=0; i < cells.length; i++) { //A la case i de positions, on mettra la position de la cellule cells[i]
	    positions[i] = cells[i].donnePosition();
	}
	return positions;
    }

    //Mthode prive analyseFormule(String expression) : appele uniquement quand on dtecte que expression POURRAIT-TRE une formule (i.e. quand la variable prive value commence par un '=')
    //Analyse expression et renvoit une structure d'arbre refltant value. Lance une exception de type FormuleException si une erreur survient dans le traitement de la formule
    private ElementFormule analyseFormule(String expression, CaseRepresentationInterne[][] representationInterne) throws FormuleException {
	//Normalement, il ne devrait jamais arriver qu'on reoive dans expression une chane vide. Si a arrive, c'est que l'expression a t mal formule
	if (expression.length() == 0) {  //On peut se trouver ici si on a une expression vide soit directement ("=") soit par rcursion ("=+(5,,,)")
	    throw new FormuleException("Erreur dans le format de la formule, on est amen  analyser une expression vide");
	}
	
	int i = 0;  //Variable de travail, servira d'indice dans le parcours de expression 
	char c = expression.charAt(0); //Variable de travail, contiendra le caractre en cours d'analyse dans expression

	//On analyse expression  la recherche des diffrents cas valides possibles pour une formule
	
	//Cas o expression pourrait tre une rfrence vers une autre cellule
	if ((c >= 'A') && (c <= 'J')) {  //Expression commence par une lettre de rfrence de cellule
	    return new ElementFormuleIdentifiantCellule(expression,this, representationInterne);   //L'appel de cette mthode peut lancer une exception de type FormuleException qu'on dlguera  la mthode appelante	    
	}
	
	//Cas o expression est un nombre
        if ((c >= '0') && (c <= '9') ||  //Si expression commence par un chiffre ...
	    (((c == '+') || (c == '-')) && ((expression.length() > 1) && (expression.charAt(1) != '(')))) { //... ou un signe (+,-) sans '(' juste aprs (sinon c'est une expression pas un simple nombre)
	    //Ici, expression pourrait tre un nombre, on crera un nouvel ElementFormuleNombre contenant la valeur de expression si on dtecte effectivement qu'on a un nombre valide
	    if (isValidNumber(expression)) {
		return new ElementFormuleNombre(expression);
	    }
	    else { //Si expression n'est pas un nombre valide, on lance une exception
		throw new FormuleException("Nombre non valide dtect comme argument de la formule");
	    }
	}

	//Cas o expression est une formule : une expression POURRAIT-TRE une formule si son deuxime caractre est une '('
	if ((expression.length() > 1) && (expression.charAt(1)) == '(') {
	    char operateur=c;     //Le type d'oprateur qu'aura la formule ('+', '-', '*', '/', ou un autre caractre non valide)
	    
	    //Ici on va parcourir la chane, s'assurer que le dernier caractre de celle-ci est bien une ')' et que cette parenthse ferme bien la '(' de l'expression principale
	    int positionChaine = 2; //On commencera au caractre 2 car les 2 premiers caractres sont dj connus : un oprateur puis une '('
	    int longueurChaine = expression.length(); //On met la longueur de la chane dans une variable pour ne pas rappeler la mthode length  chaque passage dans la boucle
	    int niveauParentheses = 1; //On dmarre au niveau 1 dans les parenthses puisque l'expression commence par une '('
    	    int debutExpression = 2;  //Indiquera l'indice du dbut d'une expression dans la chane (la premire expression commence ncessairement  la position 2, juste aprs la '(')
	    int finExpression = debutExpression; //Indiquera l'indice de la fin d'une expression.
	    boolean doitExtraireExpression=false;   //Vaudra true si le texte d'une expression doit tre extrait de la chane
	    ArrayList argumentsExpression = new ArrayList(); //On stockera le texte de chaque argument de l'expression dans ce vecteur de taille dynamique

	    //On va parcourir la chane caractre par caractre. La dernire position sera en (longueurChaine-2) pour s'assurer qu'il y aura toujours un caractre  droite du caractre courant
	    while (positionChaine < longueurChaine-1) { 		
		char c1 = expression.charAt(positionChaine);   //On va analyser le (positionChaine)ime caractre de la chane
		switch(c1) {
		case '(': niveauParentheses++; break; 
		case ')': niveauParentheses--; break;
		case ',':  //On trouve une virgule mais elle ne doit tre prise en compte que si on est au niveau 1 de parenthses
		    if(niveauParentheses == 1) {
			//On a isol une expression, on indique qu'il faut dcouper la sous-chane allant de debutExpression  finExpression-1 et l'ajouter au vecteur des expressions
			finExpression = positionChaine;  
			doitExtraireExpression=true;
		    }
		    break;
		}
		if (positionChaine == longueurChaine-2) { //Si on est  l'avant dernier caractre de la chane (juste avant la ')'), on doit marquer la fin d'une expression
		    finExpression = positionChaine+1;  //Comme il faut inclure le caractre  la position courante, finExpression doit tre l'indice du caractre suivant
		    doitExtraireExpression=true;
		}
		if (doitExtraireExpression) { 
		    argumentsExpression.add(expression.substring(debutExpression, finExpression));
		    debutExpression = finExpression+1; //L'expression suivante commence  finExpression+1 : juste aprs la ','
		    doitExtraireExpression=false;
		}
		positionChaine++;
	    }

	    //A la fin de cette boucle, on est  l'indice du dernier caractre de la chane qui doit ncessairement tre une ')' et on doit tre au niveau 1 de parenthses
	    if ((niveauParentheses != 1) || (expression.charAt(positionChaine) != ')')) {
		throw new FormuleException("Les parenthses de l'expression ne sont pas quilibres");
	    }
	    
	    //Ici on a un vecteur de String avec les arguments de l'expression. On va crer un nouveau tableau de ElementFormule qui aura la taille du vecteur
	    //et on va initialiser chaque case de tableau avec la valeur de retour de cette mme mthode qu'on va appeler rcursivement pour chaque argument prsent dans le vecteur
	    ElementFormule[] elements = new ElementFormule[argumentsExpression.size()];
	    for (int compteurElement = 0; compteurElement < elements.length; compteurElement++) {
		elements[compteurElement] = analyseFormule((String)argumentsExpression.get(compteurElement), representationInterne);
	    }
	    //Ici toutes les branches de l'arbre des lments sont construites, on va renvoyer le noeud de l'arbre : un nouvel ElementFormuleExpression
	    return new ElementFormuleExpression(operateur, elements);
	}
	
	//Si on arrive ici, la formule entre ne correspond  aucun des modles tests, elle est invalide
	throw new FormuleException("La formule entre ne correspond  aucun des modles valides");
    }



    //Mthode prive boolean isValidNumber(String expression) : renvoit true si expression est un nombre selon les critres de l'nonc, false sinon. expression doit tre NON VIDE
    private boolean isValidNumber(String expression) {
	int i; char c; //variables de travail
	String s = expression;   //On cre une variable de travail qui initialement contient la mme valeur que expression
	
	boolean estUnChiffre; //Variable de travail qui servira quand on voudra savoir si une expression est un nombre
	
	//Si au premier caractre de s on a un signe (+ ou -), on poursuit l'analyse sur la sous-chane de s sans le signe
        c = s.charAt(0);
	if ((c == '+') || (c == '-')) {	    
	    s = s.substring(1);   //On prend la sous-chane de s sans son signe
	}

	//Partie entire du nombre : on inspecte la chane tant qu'on trouve des chiffres et tant qu'on n'est pas  la fin de s
	i = 0;               //On commence l'analyse au premier caractre de la chane
	estUnChiffre = true;  //On met la variable  true pour rentrer dans la boucle
	while ((i < s.length()) && estUnChiffre) {
	    c = s.charAt(i);
	    estUnChiffre=((c >= '0') && (c <= '9'));     //c est un chiffre ssi son code ASCII est compris entre celui du '0' et celui du '9'
	    i++;
	} 

	//A la fin de la boucle, i est l'indice du premier caractre qui n'a pas encore t inspect dans la chane
	
	if (i == s.length()) {   //Si on est sorti de la boucle parce qu'on a parcouru toute la chane ...
	    if (estUnChiffre) {  //... et que le dernier caractre de la chane est un chiffre, alors l'expression est bien un nombre
		return true;
	    }
	    else { //Si le dernier caractre n'est pas un chiffre, alors la cellule est de type texte (on n'admet pas comme nombre une expression style "5.")
		return false;
	    }
	}

	//On ne se trouvera jamais ici si on a affaire  un nombre entier       

	//Si on a affaire  un nombre dcimal, i doit au moins valoir 2 (car le '.' se trouve au moins en seconde position puisqu'on n'admet pas comme nombre une expression style ".25")
	if ((i < 2) || (c != '.')) { //Dans ces deux cas, l'expression n'est pas un nombre valide
	    return false;
	}
	
	//On pourrait ici avoir un nombre dcimal, on vrifie que les caractres aprs le . sont bien des chiffres
	estUnChiffre = true;  //On met la variable  true pour rentrer dans la boucle
	while ((i < s.length()) && estUnChiffre) {
	    c = s.charAt(i);
	    estUnChiffre=((c >= '0') && (c <= '9'));     //c est un chiffre ssi son code ASCII est compris entre celui du '0' et celui du '9'
	    if (!estUnChiffre) {   //vu qu'on est aprs le symbole ".", on est certain que si on rencontre un caractre non chiffre, alors expression n'est pas un nombre
		return false;
	    }
	    i++;
	}

	//Si on est ici, on a un nombre dcimal bien format
	return true;
    }

    //Mthode Coordonnees[] setValue(String expression, CaseRepresentationInterne representationInterne) : Donne  la case la valeur expression,
    //dtecte si expression est un type math ou texte et rgle le type de la case en fonction.
    //On aura besoin d'une rfrence vers le tableau complet de la reprsentation interne au cas o on devrait modifier la valeur d'une cellule dont dpendent d'autres cellules pour s'valuer
    //Renvoit un vecteur de Coordonnees qui contient les coordonnes des les cellules qui devront tre redessines dans la grille suite  un changement de la valeur de cette case
    //Lance une FormuleException si une rfrence circulaire est dtecte quand on cherche les cellules lies a this
    public Coordonnees[] setValue(String expression, CaseRepresentationInterne[][] representationInterne) throws FormuleException {
	//On commence par supprimer toutes les rfrences vers this que pourraient avoir les autres cellules de la reprsentation interne : si la case tait une formule,
	//elle pouvait contenir des rfrences vers d'autres cellules et ces cellules savaient qu'elles taient rfrences par this
	delieDesAutresCellules(representationInterne);

	value = expression;      //On met l'expression dans value, ensuite on dterminera son type

	if (value.length() == 0) { //expression est une chane vide -> on lui met le bon type et on sort de la mthode
	    type = CELLULE_VIDE;
	    peutEtreEvaluee = false;  //On ne permet pas l'valuation d'une cellule vide 
	    formule = null;  //On s'assure que le garbage collector librera toute la mmoire si on passe d'une case math avec beaucoup de formules  une case vide
	}
	//Si s n'est pas vide, on cherche  savoir si s est un nombre
	else if (isValidNumber(expression)) {  //Si expression est dtecte comme un nombre, la cellule est de type math
	    type = CELLULE_MATH;
	    peutEtreEvaluee = true;        //Une cellule contenant un nombre pourra tre value (on renverra ce nombre)
	    formule = null;                //C'est en fonction de (formule == null) qu'on distinguera une case math contenant une formule d'une case math contenant juste un nombre
	}
	//Sinon, on regarde si expression est une formule
	else if (expression.charAt(0) == '=') {  //On a une formule si le premier caractre de expression est '=', on va l'analyser et la stocker dans formule
	    type = CELLULE_MATH;  //On sait qu'on a une formule, reste  savoir si elle est valide
	    try {  //On va essayer d'analyser la formule
		formule = analyseFormule(expression.substring(1), representationInterne);  //On demande l'analyse de expression sans son signe '=' devant
		peutEtreEvaluee = true;     //Ici, on est certain que la formule a t analyse correctement et qu'elle pourra tre value
	    }
	    catch (FormuleException e) { //Quelque-chose de mal s'est produit lors de l'analyse de la formule et de la construction de l'arbre
		//System.err.println(e.getMessage());
		formule = null;
		type = CELLULE_ERRONEE;  //On met une formule dont le texte est erron au type CELLULE_ERRONEE
	    }
	}
	else { //Sinon on a une cellule de type texte
	    type = CELLULE_TEXTE;
	    peutEtreEvaluee = false;  //On ne permet pas l'valuation d'une case texte
	    formule = null; //On s'assure que le garbage collector librera toute la mmoire de l'arbre de la formule si par ex passe d'une case math avec beaucoup de formules  une case texte
	}
	return donnePositionCellulesLiees();   //on essaye de renvoyer un vecteur contenant la position des cellules lies  this mais ceci peut causer des problmes en cas de dfinition circulaire
    }

    //Mthode getEditorText() : renvoit une forme directement ditable de la cellule instancie (le texte que contiendra l'diteur)
    public String getEditorText() {
	return value;
    }

    //Mthode prive testeDefinitionCirculaire(HashSet parcours) throws FormuleException : s'assure qu'on n'est pas entrain de boucler  l'infini ( cause d'une dfinition circulaire dans une formule)
    //Si une dfinition circulaire est dtecte, la mthode lance une exception de type FormuleException
    private void testeDefinitionCirculaire(HashSet parcours) throws FormuleException {
	//Si une rfrence vers cette cellule est dj prsente dans l'ensemble des rfrences de cellules qui composent parcours, on a une dfinition circulaire dans la formule
	if (!(parcours.add(this))) {   //La mthode add de HashSet renverra false si this est dj prsent dans l'ensemble
	    throw new FormuleException("Dfinition circulaire dtecte dans la formule");
	}
    }

    //Mthode public double evalue() : renvoit l'valuation de la case instancie : si une case contient un nombre, on renvoit celui-ci sous forme de double, si c'est une formule
    //on renvoit son valuation sous forme de double, sinon on lance une exception. Cette mthode a besoin d'une rfrence vers la reprsentation interne complte : representationInterne
    public double evalue(CaseRepresentationInterne[][] representationInterne, HashSet parcours) throws FormuleException {
	//On commence par s'assurer qu'on n'a pas de dfinition circulaire
	testeDefinitionCirculaire(parcours);    //Si une dfinition circulaire est dtecte, une exception sera lance ici
	
	if ((type == CELLULE_VIDE) || (type == CELLULE_TEXTE) || (type == CELLULE_ERRONEE)) { //Dans ces trois cas, une valuation est impossible
	    throw new FormuleException("Tentative d'valuation d'une case non math");
	}

	//Ici on est sr d'avoir une case math, reste  distinguer 2 types : un nombre ou une formule
	if (formule == null) { //Si la case est un nombre, on convertit simplement value en double
	    return Double.parseDouble(value);
	}

	//Ici, on a une formule dont on renvoit l'valuation
	return formule.evalue(representationInterne, parcours);
    }

    //Mthode public String getValue() : renvoit la reprsentation "externe" de la case instancie (le texte qu'on va afficher dans la grille).
    //Cette mthode aura besoin d'une rfrence vers le tableau complet de la reprsentation interne (au cas o on aurait des rfrences de cellules  traiter)
    //Elle peut lancer une FormuleException si une dfinition circulaire ou une rfrence vers une cellule invalide est dtecte
    public String getExternalRepresentation(CaseRepresentationInterne[][] representationInterne) {
	if ((type == CELLULE_TEXTE) || (type == CELLULE_VIDE)) { //On a une case texte ou une case vide, il faut renvoyer value sans rien faire d'autre
	    return value;
	}
	else if (type == CELLULE_ERRONEE) {    //On a une cellule contenant une formule invalide
	    return TEXTE_CASE_ERRONEE;
	}

	//Ici, on est ncessairement dans le cas d'une cellule de type math
	else if (peutEtreEvaluee) {  //Si la case est une case math valide
	    if (formule == null) { //Si formule est null, cette case est un simple nombre, il suffit de renvoyer value
		return value;
	    } 

	    //On renvoit la conversion en String de l'valuation de la case, sauf si on rencontre une cellule qui n'est pas valuable lors de l'valuation d'une formule, alors exception
	    try{
		return Double.toString(evalue(representationInterne, new HashSet()));   //On essaye d'valuer la case
	    }
	    catch(FormuleException e) { //Formule pas valuable ou rfrence circulaire ?
		//Dans ce cas, la formule est valide mais n'est pas valuable, pour une raison externe  la case
		//System.err.println(e.getMessage());
		return new String(TEXTE_CASE_ERRONEE);  
	    } 
	}

	//Cas d'une cellule ne pouvant pas tre value
	return new String(TEXTE_CASE_ERRONEE);
    }


    //Mthode getType() : renvoit le type de la case instancie
    public int getType() {
	return type;
    }
}

		

	
