//ETAPE 4

import javax.swing.*;
import java.awt.*;
import javax.swing.table.*;
           
import java.io.*;               //Dfinit les exceptions du type IOException

public class Grille extends JTable {

    //Les variables d'instance
    protected ZoneEdition editeur;                               //La zone dans laquelle on pourra modifier le contenu d'une cellule
    protected RepresentationInterne representationInterne;       //Reprsentation intere, gr par la classe RepresentationInterne

    //Les sous-classes internes  Grille  
    private class RenderCell extends JLabel implements TableCellRenderer { //Dessine les cellules "normales" du tableau

	private int ligneEnCours;    //Le numro de la ligne (dans la reprsentation interne) de la case qu'on reprsente
	private int colonneEnCours;  //Le numro de la colonne (dans la reprsentation interne) de la case qu'on reprsente
      

	//CONSTRUCTEUR
	public RenderCell() {
	    setOpaque(true);  //Permet d'utiliser une couleur de fond pour la cellule 
	}
	
	//On implmente cette mthode de l'interface TableCellRenderer pour personaliser la faon dont la cellule est dessine
	public Component getTableCellRendererComponent (JTable table, Object obj, 
							boolean isSelected, boolean hasFocus,
							int row, int column) {
	    
	    double minimalWidth;                   //Donnera la largeur minimale qu'il faut  ce composant pour que son texte soit affich en entier
	    int rowWidth;                          //Donnera la largeur de la colonne dans laquelle ce JLabel ira se placer 
	    boolean truncate;                      //Vaudra true s'il faut tronquer le texte
	    boolean textTruncated;                 //Vaudra true si le texte a t tronqu
	    
	    //On calcule la ligne et la colonne  adresser dans la reprsentation interne. Il faut retirer une unit  row et column car la zone ditable commence en (1,1)
	    ligneEnCours = row-1;
	    colonneEnCours = column-1;
	   
	    //La valeur qu'il faudra afficher : on la demande  la representation interne	    
	    String valeur = representationInterne.getExternalRepresentationAt(ligneEnCours, colonneEnCours);

	    //Le type de la cellule qu'il faudra afficher (soit "math" soit "texte")
	    int type = representationInterne.getTypeAt(ligneEnCours, colonneEnCours);
	    if (type == CaseRepresentationInterne.CELLULE_MATH) {         //Le texte des cases "math" sera aligne  droite
		setHorizontalAlignment(SwingConstants.RIGHT); 
	    }
	    else if (type ==  CaseRepresentationInterne.CELLULE_TEXTE) {  //Le texte des cases "texte" sera aligne  gauche  
		setHorizontalAlignment(SwingConstants.LEFT);
	    }
	       
	    rowWidth = table.getColumnModel().getColumn(column).getWidth();   //dans rowWidth, on met la largeur disponible dans la colonne
	    textTruncated = false;           //Initialement on suppose que le texte peut tre affich intgralement dans sa cellule
	    do {  //On compare rowWidth et minimalWidth
		setText(valeur);   //On commence par donner au JLabel le texte de valeur, ensuite on regarde si le texte est suffisamment petit pour ne pas tre tronqu dans la grille
		
		minimalWidth = getPreferredSize().getWidth();   //dans minimalWidth, on met la taille que souhaiterait le JLable (this) pour afficher l'entiret de son texte

		//Tant que la taille minimale ncessaire pour afficher le composant en entier est plus grande que l'espace disponible dans la colonne, on supprime le dernier caractre de valeur		
	        truncate  = (minimalWidth > rowWidth);

		if (truncate) { 
		    textTruncated = true;                               //On indique que le texte a t tronqu
		    valeur = valeur.substring(0, (valeur.length()-1));  //Supprimer le dernier caractre de valeur
		}
	    } while (truncate);
	    if (textTruncated) {   //L'nonc demande que si on doit tronquer le texte, l'arrire plan de la cellule devient bleu
		if (!isSelected) {  
		    setBackground(Color.blue);  //Fond bleu standard si la cellule n'est pas slectionne
		}
		else {
		    setBackground(new Color(0x000080));  //Fond bleu plus fonc pour bien voir les cellules tronques slectionnes
		}
		setForeground(Color.white);
	    }
	    else {  //Sinon le texte a pu tre affich intgralement dans la cellule -> texte noir sur fond soit blanc si la cellule n'est pas slectionne, sur fond gris clair sinon
		setForeground(Color.black);
		if (!isSelected) {
		    setBackground(Color.white);
 		}
		else {
		    setBackground(Color.lightGray);
		}
		    
	    }
	    return this;   //On renvoit le label personnalis comme renderer
	}
    }

    private class RenderButton extends JButton implements TableCellRenderer {  //Dessine les boutons  la ligne 0 et  la colonne 0

	//On implmente cette mthode de l'interface TableCellRenderer pour personaliser la faon dont la cellule est dessine
	public Component getTableCellRendererComponent (JTable table, Object obj, 
							boolean isSelected, boolean hasFocus,
							int row, int column) {
	    if ((row == 0) && (column > 0)) { //On est dans le cas o il faut dessiner les boutons avec les numros des colonnes (A, B, ...)
		//On fait en sorte que le texte du bouton corresponde au numro de la colonne (A=0, B=1, ...)
		setText((new Character((char)('A'+(column-1)))).toString());			
	    }
	    else if ((column == 0) && (row > 0)) { //On est dans le cas o il faut dessiner les boutons avec les numros des lignes (0, 1, ...)
		
		//Le bouton contiendra le numro de la ligne
		setText(Integer.toString(row-1));   //row-1 parce que le bouton de la ligne 0 de la zone ditable est  la ligne 1 sur la grille
	    }
	    //Si on n'est dans aucun de ces cas, alors il s'agit du bouton en haut  gauche qui n'a aucun texte

	    return this;            //On renvoit le bouton
	}
    }

    private class EditeurCellule extends AbstractCellEditor implements TableCellEditor { //Cette classe sera instancie chaque fois qu'il faudra modifier une case non bouton du tableau	
	JLabel texte = new JLabel();

	int colonneEnCours;  //Le numro de la colonne (dans la reprsentation interne) de la case qu'on dite
	int ligneEnCours;    //Le numro de la ligne (dans la reprsentation interne) de la case qu'on dite

	//CONSTRUCTEUR
	public EditeurCellule() { 
	    setOpaque(true);  //Permet d'utiliser une couleur de fond pour la cellule
	}

	public Component getTableCellEditorComponent (JTable table, Object value, 
						      boolean isSelected,
						      int row, int column) {

 
	    colonneEnCours = column - 1; ligneEnCours = row - 1;    //ATTENTION que la zone utile de la grille commence en 1,1 -> il faut retirer une unit pour une adresse correcte !!!!!
	    
	    String s=representationInterne.getEditorTextAt(ligneEnCours, colonneEnCours);  	    
	    
	    editeur.setText(s);            //Le texte de la zone d'dition devient la reprsentation interne de la case qu'on veut diter
	    editeur.requestFocus();        //L'diteur devient le composant qui a le focus pour pouvoir diter facilement les cellules
	    editeur.selectAll();           //On slectionne tout le texte de l'diteur pour que quand on tape quelque-chose au clavier le contenu de l'diteur soit directement remplac par le nouveau

	    //Pendant l'dition, le texte de la case reste ce qu'il tait avant l'dition, le texte est align  gauche et la case est en noir
	    JLabel caseEnEdition = new JLabel(s);
	    caseEnEdition.setHorizontalAlignment(SwingConstants.LEFT);
	    caseEnEdition.setOpaque(true);
	    
	    caseEnEdition.setForeground(Color.white);
	    caseEnEdition.setBackground(Color.black);

	    return caseEnEdition;
	}

	public Object getCellEditorValue(){   //Cette mthode est appele quand on a fini d'diter une cellule, par ex quand on clique dans une autre
	    placeDansRepresentationInterne(editeur.getText(), ligneEnCours, colonneEnCours);	     	    
	    return null;                      //On n'a besoin de donner aucune valeur puisque, comme la reprsentation interne va changer, le renderer se chargera de l'affichage
	}
    } 

    //CONSTRUCTEUR : a besoin d'une rfrence vers un composant de type ZoneEdition qu'il utilisera lorsqu'on voudra modifier le contenu d'une case
    public Grille(ZoneEdition z) { 
	//On cre une grille plus grande d'une ligne et d'une colonne afin d'utiliser la colonne 0 et la ligne 0 pour en faire des boutons
	super(RepresentationInterne.NB_LIGNES+1, RepresentationInterne.NB_COLONNES+1);   
	
	//Paramtres pour le look & feel de la grille
	setCellSelectionEnabled(true); //On permet la slection individuelle des cellules
	setRowHeight(20);  //Toutes les lignes du tableau auront la hauteur 20
	setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);   //Permettre la slection d'un intervalle de cellules  la souris dans la grille

	//Initialiser la reprsentation interne
	representationInterne = new RepresentationInterne();

	editeur = z;  //On associe l'diteur de cellule de la grille  la ZoneEdition passe en paramtre

	//On slectionne la premire cellule ditable (en position (1,1))
	setRowSelectionInterval(1, 1);
	setColumnSelectionInterval(1, 1);
	setEditingRow(1);
	setEditingColumn(1);

	repaint();

    }
    
    //Mthode load (String fileName) : charge le tableau fileName sur le disque dur et on remplit la grille avec celui-ci
    public void load (String fileName) throws IOException, FormatException {
	  representationInterne.load(fileName);
	  repaint();    //La reprsentation interne a chang, on redessine la grille
    }

    //Mthode save (String fileName) : enregistre le contenu de la grille dans le fichier fileName
    public void save (String fileName) throws IOException {
	representationInterne.save(fileName);
    }

    //Mthode representationInterneModifiee() : renvoit true si la reprsentation interne de cette grille a subi des modifications
    public boolean representationInterneModifiee() {
	return representationInterne.estModifiee();
    }

    //Mthode placeDansRepresentationInterne(String s, int i, int j) : Place s  la case (i,j) de la reprsentation interne. 
    //Redessine les cellules du vecteur de Coordonnees que renvoit setValueAt
    public void placeDansRepresentationInterne(String s, int i, int j) {
	try{ //La mthode setValueAt de representationInterne peut lancer une FormuleException s'il s'avre impossible de dterminer quelles cases il faut redessiner
	    Coordonnees[] aRedessiner = representationInterne.setValueAt(s, i, j);
	    //On va parcourir le vecteur aRedessiner et demander  la grille de redessiner toutes les cellules rfrences par celui-ci
	    for (int k = 0; k < aRedessiner.length; k++) {
		//Attention que la cellule (i,j) de la reprsentation interne est  la position (i+1,j+1) dans la grille
		repaint(getCellRect(aRedessiner[k].getLineNumber()+1, aRedessiner[k].getColumnNumber()+1, true));  
	    }
	}
	catch(FormuleException e) {  //Il y a eu un problme et on ne sait pas quoi redessiner dans la grille, alors on redessine tout
	    repaint();
	}
    }

    //---------------------------------------------------OPRATIONS D'DITION--------------------------------------------------------------
    //Mthode copySelectedArea() : copie le texte des cellules slectionnes dans le presse-papiers
    public boolean copySelectedArea() {
	String[][] tab;  //Le tableau qu'on mettra dans le presse-papiers (seules les cellules correspondant  une zone slectionne seront non null dans ce tableau)

	//On va rechercher les coordonnes de la zone slectionne
	int premiereLigne = getSelectedRow();
	int derniereLigne = premiereLigne + getSelectedRowCount() - 1; //Pour se rappeler  quoi sert le "-1", penser au cas o une seule ligne est slectionne
	if (premiereLigne == 0) {   //Si premiereLigne = 0 on a slectionn la ligne avec les boutons
	    premiereLigne = 1;  //On enlve la slection de cette ligne et on commence  la ligne 1
	    if (derniereLigne < premiereLigne) {  //Dans ce cas, on n'aurait slectionn que la ligne de boutons -> on ne sait rien copier
		return false;   
	    }
	}
	int premiereColonne = getSelectedColumn();
	int derniereColonne = premiereColonne + getSelectedColumnCount() - 1; //Pour se rappeler  quoi sert le "-1", penser au cas o une seule colonne est slectionne
	if (premiereColonne == 0) {  //Si premiereColonne = 0 on a slectionn la colonne avec les boutons
	    premiereColonne = 1;
	    if (derniereColonne < premiereColonne) {  //Dans ce cas, on n'aurait slectionn que la colonne de boutons -> on ne sait rien copier
		return false;  
	    }
	}
	//On cre un tableau de la mme taille que la zone slectionne (pour se rappeler  quoi servent les "+1" penser aux cas o une seule ligne/colonne est slectionne)
	tab = new String[derniereLigne - premiereLigne + 1][derniereColonne - premiereColonne + 1];

	//On parcourt la slection
	for (int i = premiereLigne; i <= derniereLigne; i++) { 
	    for (int j = premiereColonne; j <= derniereColonne; j++) {
		//Pour chaque cellule, on cre la case correspondante de tab en lui donnant le texte de l'diteur de la cellule de la reprsentation interne correspondante
		tab[i-premiereLigne][j-premiereColonne] = new String(representationInterne.getEditorTextAt(i-1,j-1));  //Attention que la zone ditable commence en (1,1) dans la grille
	    }
	}
	//Ici tab est initialis avec le contenu de la slection, on envoit tab dans le presse-papiers
	Clipboard.put(tab);

	return true;
    }
    
    //Mthode pasteAtEditingCell() : colle le contenu du presse-papiers en commenant  la cellule en dition. Renvoit false si on n'a slectionn aucune cellule ditable (i.e. si on a cliqu sur un bouton)
    public boolean pasteAtEditingCell() {
        String[][] tab;   //On placera le contenu du presse-papiers ici

	//Calculer les coordonnes de la cellule marquant le coin suprieur gauche de la zone  coller dans la reprsentation interne
	int premiereLigne = getEditingRow()-1;  //Il faut retirer une unit puisque la zone ditable de la grille commence en (1,1)
	int premiereColonne = getEditingColumn()-1;
	//Si les coordonnes de la cellule en dition sont invalides, on renvoit false
	if ((premiereLigne < 0) || (premiereColonne < 0)) {
	    return false;   //Impossible de coller car on n'a slectionn aucune cellule ditable
	}
	
	tab = Clipboard.get();           //On demande le contenu du presse-papiers

	//On va parcourir tab lment par lment et replacer chaque lment dans la reprsentation interne  la bonne place, sauf si on sort de la reprsentation interne, l on ne fait rien
	for (int i = premiereLigne;  //On commence  crire dans la reprsentation interne  la premire ligne calcule
	     (i < RepresentationInterne.NB_LIGNES) &&  //Prgardien pour viter d'crire hors des limites du tableau de la reprsentation interne
		 (i - premiereLigne < tab.length);  //On arrte la boucle quand on a parcouru toutes les lignes de tab
	     i++) {  

	    for (int j = premiereColonne; 
		 (j < RepresentationInterne.NB_COLONNES) && 
		     (j - premiereColonne < tab[i-premiereLigne].length); 
		 j++) {

		placeDansRepresentationInterne(tab[i-premiereLigne][j-premiereColonne], i, j);
	    }
	}
	repaint();

	//On change le texte de l'diteur pour qu'il corresponde au contenu de la reprsentation interne aux coordonnes de la cellule en dition
	editeur.setText(representationInterne.getEditorTextAt(premiereLigne, premiereColonne));
	
	editCellAt(premiereLigne+1, premiereColonne+1);            //Ceci a pour effet de redessiner la cellule qui a servi de coin suprieur gauche  la zone de collage
	return true;   //On signale que le collage a t effectu avec succs

    }

    //Mthode deleteSelectedArea() : efface le contenu de la zone slectionne
    public void deleteSelectedArea() {
	//On va rechercher les coordonnes de la zone slectionne
	int premiereLigne = getSelectedRow();
	int derniereLigne = premiereLigne + getSelectedRowCount() - 1; //Pour se rappeler  quoi sert le "-1", penser au cas o une seule ligne est slectionne
	if (premiereLigne == 0) {   //Si premiereLigne = 0 on a slectionn la ligne avec les boutons
	    premiereLigne = 1;  //On enlve la slection de cette ligne et on commence  la ligne 1
	    if (derniereLigne < premiereLigne) return;   //Dans ce cas, on n'aurait slectionn que la ligne de boutons -> rien  faire
	}
	int premiereColonne = getSelectedColumn();
	int derniereColonne = premiereColonne + getSelectedColumnCount() - 1; //Pour se rappeler  quoi sert le "-1", penser au cas o une seule colonne est slectionne
	if (premiereColonne == 0) {  //Si premiereColonne = 0 on a slectionn la colonne avec les boutons
	    premiereColonne = 1;
	    if (derniereColonne < premiereColonne) return;  //Dans ce cas, on n'aurait slectionn que la colonne de boutons -> rien  faire
	}

	//On parcourt la slection
	for (int i = premiereLigne; i <= derniereLigne; i++) { 
	    for (int j = premiereColonne; j <= derniereColonne; j++) {
		//On efface chaque cellule
		placeDansRepresentationInterne("",i-1,j-1);  //Attention que la zone ditable commence en (1,1) dans la grille -> changement de coordonnes
	    }
	}
	    
	//On efface aussi le texte de la zone d'dition
	editeur.setText("");

	//On redessine la grille
	repaint();

	//La reprsentation interne a chang, redessiner la cellule en dition qui maintenant est vide
	editCellAt(getSelectedRow(), getSelectedColumn());   //Ceci a pour effet d'aussi redessiner la cellule en dition
    }
    

    //Mthode cutSelectedArea() : copie le contenu de la zone slectionne puis efface son contenu. Renvoit true si elle a effectivement coup qqch, false si la slection tait invalide
    public boolean cutSelectedArea() {
	//Une opration "couper" est quivalente  une opration "copier" suivi de l'effacement de la cellule
	if (copySelectedArea()) { //Si on a su copier quelque-chose (donc que quelque-chose est dj slectionn)
	    deleteSelectedArea();
	    return true;
	}
	//Si on est ici, c'est qu'on n'a rien copi (slection invalide)
	return false;
    }	
   
    //---------------------------------------------MTHODES HRITES DE JTable-------------------------------------------------------------
    //On va utiliser un renderer diffrent en fonction de la position de la cellule qu'on voudra dessiner 
    public TableCellRenderer getCellRenderer (int row, int column) { 
	//La premire colonne et la premire ligne de la grille sont rserves aux boutons du tableau, le reste, c'est des cellules "normales"
	if ((row > 0) && (column > 0)) {   //Devons-nous instancier un renderer de cellule ?
	    return new RenderCell();
	}
	else {  //Sinon on doit instancier un renderer de bouton
	    return new RenderButton();
	}
    }

    //L'diteur associ  la grille : edition impossible si on clique sur un bouton, afficher la reprsentation interne sinon    
    public TableCellEditor getCellEditor (int row, int column) {
	//La premire colonne et la premire ligne de la grille sont rserves aux boutons du tableau, le reste : cellules ditables
	if ((row > 0) && (column > 0)) {   //Devons-nous instancier un diteur de cellule ?
	    return new EditeurCellule();        
	}
	else {  //Sinon on est sur un bouton, dition impossible -> pas d'diteur
	    return null;
	}	
    }        

    
}
    
  
