// ETAPE 4
// Nouvelle classe.

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class PanneauGrilleEditeur extends JPanel implements KeyListener
{

    //Cette sous classe interne sert  mettre un filtre qui n'affiche que les fichiers d'extension .stp 
    //dans les botes de dialogue ouvrir et enregistrer sous
    public class FiltreSTP extends javax.swing.filechooser.FileFilter //on doit crire le chemin complet de la classe qu'on tend pour viter les rfrences ambiges car il existe 2 classes FileFilter dans les packages qu'on importe
    {
	//Mthode hrite accept(File f) : renvoit true s'il faut rfrencer le fichier f dans la liste des fichiers de la bote de dialogue
	public boolean accept(File f)
	{
	    if (f.isDirectory())
		{
		    return true; //On doit pouvoir visualiser les rpertoires
		}
	    String suffixe = null;
	    String s = f.getName();
	    int i = s.lastIndexOf('.'); //i contiendra l'index du dernier "."
	    if (i > 0 && i < s.length() -1) //Si il y a au moins un caractre aprs le point
		{
		    //Retourne la sous-chaine situe aprs le dernier caractre "." et convertie en minuscules
		    suffixe = s.substring(i + 1).toLowerCase(); 
		}
	    return (suffixe != null && suffixe.equals("stp"));
	}
	
	//mthode getDescription() : renvoit le nom du filtre (apparatra dans la bote de dialogue de slection du fichier)
	public String getDescription()
	{
	    return "Fichiers Super Tableur Pro";
	}
    }
    


    private Grille grille;
    private ZoneEdition zoneEdition;
    private String fichierEnEdition = "";
    private static int nombreInstanciations = 0;
 
    public PanneauGrilleEditeur()
    {
	this.setLayout(new BorderLayout());
	zoneEdition = new ZoneEdition();
	zoneEdition.addKeyListener(this);
	grille = new Grille(zoneEdition);
	this.add(zoneEdition,BorderLayout.NORTH);
	this.add(grille,BorderLayout.CENTER);
	nombreInstanciations++;
	fichierEnEdition="SansNom"+nombreInstanciations; //Tant qu'on aura pas enregistr ou sauvergard un fichier, la variable fichierEnEdition commencera par "SansNom".
    }

    public String donneTitre()
    {
	return fichierEnEdition;
    }

    public String donneTitreRaccourci() //Renvoit la chaine fichierEnEdition  laquelle on retire l'arborescence du nom de fichier.
    {
	int i = fichierEnEdition.lastIndexOf(File.separatorChar); //i contiendra l'index du dernier caractre de sparation dans l'arborescence d'un fichier
	String titreRaccourci = fichierEnEdition;
	if (i > 0)
	    {
		titreRaccourci = fichierEnEdition.substring(i + 1) ;
	    }
	return titreRaccourci;
    }
       

    //Mthode chargement():
    //Affiche une bote de dialogue de slection de fichier et ragit correctement aux actions de l'utilisateur
    public void chargement()
    {
	try
	    {			       
		//On cre une bote de dialogue de choix de fichier
		JFileChooser fc = new JFileChooser("./");  //On ouvre la bote de dialogue dans le dossier courant
		fc.setDialogTitle("Chargement");
		fc.setApproveButtonText("Charger");
		FiltreSTP fstp = new FiltreSTP();
		fc.addChoosableFileFilter(fstp); //On ajoute  la bote de dialogue un filtre pour les fichiers .stp
		int retour = fc.showOpenDialog(this);
		if (retour == JFileChooser.APPROVE_OPTION) 
		    {
			//On a cliqu sur Charger			
			String nomFichier = fc.getSelectedFile().toString();
			File f = new File(nomFichier);
			if (f.exists())     //On s'assure que le fichier choisi existe bien rellement
			    {
				//On demande explicitement que la cellule slectionne pour l'dition soit la cellule (0,0)
				//(qui n'existe pas) sinon si une cellule est slectionne, son ancien contenu reste dans l'diteur 
				//et la reprsentation interne prendra une mauvaise valeur aux coordonnes de la cellule qui tait en
				//dition au moment o on voudra diter une cellule diffrente de celle slectionne
				grille.editCellAt(0, 0);
				grille.setRowSelectionInterval(0, 0);
				grille.setColumnSelectionInterval(0, 0);   
				grille.setEditingRow(0);
				
				zoneEdition.setText("");//On efface aussi le texte de la zone d'dition (ce n'est pas indispensable mais c'est plus beau)
				
				grille.load(nomFichier);   
				
				//Ici, la reprsentation interne a t modifie et contient les valeurs du fichier
				
				//On reslectionne la cellule (1,1) car si on reste sur la cellule (0,0) le programme peut planter 
				//si on fait flche gauche ou flche haut (on pourrait sortir de la grille soit par la gauche, soit par 
				//le haut) 
				grille.setColumnSelectionInterval(1, 1);   
				grille.setRowSelectionInterval(1, 1);
				fichierEnEdition = nomFichier;
			    }
			else
			    JOptionPane.showMessageDialog(this, "Le fichier n'existe pas", "Erreur", JOptionPane.WARNING_MESSAGE);
		    }
	    }
	catch (IOException e)
	    {
		JOptionPane.showMessageDialog(this,"Une erreur d'entre-sortie est survenue: \n"+e.getMessage(),
					      "Erreur d'entre-sortie", JOptionPane.WARNING_MESSAGE);
	    }
	catch (FormatException e)
	    {
		JOptionPane.showMessageDialog(this, e.getMessage(), "Erreur de format de fichier", JOptionPane.WARNING_MESSAGE);
	    }
    }
    
    //Mthode enregistrer() : enregistre le fichier en dition, si on a dj attribu un nom. Sinon, on appelle 
    //la mthode enregistrerSous(). Renvoie true si il y a un nom attribu au fichier, sinon renvoie la valeur de enregistrerSous()
    public boolean enregistrer()
    {
	if( !(fichierEnEdition.startsWith("SansNom")) )  //Si on a attribu un nom au fichier (aucun systme d'exploitation ne commence le chemin d'arborescence d'un fichier par "SansNom" ...)
	    {
    		try
		    {
    			grille.save(fichierEnEdition);
			return true;            //l'enregistrement a t effectu avec succs
		    }
    		catch (IOException e)
		    {
    			JOptionPane.showMessageDialog(this,"Une erreur d'entre-sortie est survenue :\n"+e.getMessage(),
						      "Erreur d'entre-sortie", JOptionPane.WARNING_MESSAGE);
			return false;           //l'enregistrement n'a pas t effectu
		    }
	    }
    	else  
	    {
		return enregistrerSous();
	    }
    }

    //Mthode enregistrerSous():ouvre une bote de dialogue de slection de fichier pour donner un nom au fichier, puis enregistre le 
    //fichier. Renvoie true si on a cliqu sur 'Enregistrer', sinon false
    public boolean enregistrerSous() 
    {
	try
	    {
		//On cre une bote de dialogue de choix de fichier
		JFileChooser fc = new JFileChooser("./"); //On ouvre la bote de dialogue dans le dossier courant
		FiltreSTP fstp = new FiltreSTP();
		fc.addChoosableFileFilter(fstp); //On ajoute  la bote de dialogue un filtre pour t dles fichiers .stp
		fc.setDialogTitle("Enregistrer sous ...");
		fc.setApproveButtonText("Enregistrer");
		int retour = fc.showOpenDialog(this);
		if (retour == JFileChooser.APPROVE_OPTION)
		    {
			// On a cliqu sur Enregistrer
			String nomFichier = fc.getSelectedFile().toString();
			fichierEnEdition = nomFichier;
			//ajouter une extension stp au nom du fichier si il n'y en pas dj une ET si le filtre .stp est slectionn
			if (!(nomFichier.endsWith(".stp")) && (fc.getFileFilter() instanceof FiltreSTP))
			    nomFichier = (nomFichier + ".stp");
			File f = new File(nomFichier);
			if (!(f.exists()))           //si un fichier du mme nom n'existe pas dj, on sauvegarde
			    {			
				grille.save(nomFichier);
				
				return true;         //l'enregistrement a t fait avec succs
			    }
			else
			    { //Si le fichier existe dj, on demande si l'utilisateur dsire l'craser
				int reponse = JOptionPane.showConfirmDialog(this, 
									    "Ce fichier existe dj.  Voulez-vous l'craser ?",
									    "Confirmation", JOptionPane.YES_NO_OPTION);
				if (reponse == JOptionPane.YES_OPTION) //Si l'utilisateur confirme, on l'crase
				    {
					grille.save(nomFichier);
					return true;                  //l'enregistrement a t effectu avec succs
				    }
				else
				    {
					return false;                 //on signale que l'enregistrement n'a pas effectu
				    }
				
			    }
		    }
		else
		    {
			return false;                                 //si on ne valide pas l'enregistrement
		    }
	    }
	catch (IOException e)
	    {
		JOptionPane.showMessageDialog(this,"Une erreur d'entre-sortie est survenue: \n"+e.getMessage(),
					      "Erreur d'entre-sortie", JOptionPane.WARNING_MESSAGE);
		return false;       //on signale que l'enregistrement n'a pas effectu
	    }
    }

    public boolean operationsFermeture() // Renvoit false si l'utilisateur a annul l'enregistrement
    {
	if (grille.representationInterneModifiee())   //Si on a modifi la grille on demande s'il faut l'enregistrer avant de quitter
	    {
		int retour=JOptionPane.showConfirmDialog(grille, "Voulez-vous enregistrer le fichier "+fichierEnEdition+" avant de le fermer?",
							 "Confirmation", JOptionPane.YES_NO_CANCEL_OPTION, 
							 JOptionPane.QUESTION_MESSAGE);
		if (retour==JOptionPane.YES_OPTION)
		    {
			if (enregistrer())
			    return true;
			else
			    return false; //On signale que l'utilisateur a annul l'enregistrement
		    }	
		else if (retour==JOptionPane.CANCEL_OPTION)
			 return false;	    
	    }
	return true;
    }

    public boolean copier()
    {
	return grille.copySelectedArea();
    }

    public boolean couper()
    {
	return grille.cutSelectedArea();
    }

    public void coller()
    {
	if (!grille.pasteAtEditingCell()) //Si on a pas su coller
	    {	
		JOptionPane.showMessageDialog(this,"Opration de collage impossible","Erreur", JOptionPane.WARNING_MESSAGE);
	    }

    }

    public void keyTyped(KeyEvent e)
    {
    }

    public void keyReleased(KeyEvent e)
    {
    }

    //Gestion des vnements lis aux touches du clavier
    public void keyPressed(KeyEvent e)
    {
	int key = e.getKeyCode();            //On regarde quelle touche du clavier a t utilise
	int modifiers = e.getModifiers();    //On regarde aussi quel modifier a t utilis (un modifier est ctrl, alt, shift, ...)
	int sr = grille.getSelectedRow();    //On aura besoin de la colonne et de la ligne en cours
	int sc = grille.getSelectedColumn();
    
	if ((key == KeyEvent.VK_ENTER) && (modifiers == 0))  
	    {
		//Quand on presse enter on se dplace d'une cellule vers le bas 
		//Si on est en bas d'une colonne, on va dans la premire case de la colonne  droite de la colonne en cours
		//Si on est dans la dernire case de la dernire colonne, on va dans la premire case de la premire colonne   
		if (sr < RepresentationInterne.NB_LIGNES)     //On n'est pas  la dernire range -> on descend d'une range sur la mme colonne
		{
			grille.editCellAt(sr + 1, sc);
			grille.setRowSelectionInterval(sr + 1,sr + 1);
			grille.setColumnSelectionInterval(sc, sc);   
			grille.setEditingRow(sr + 1);
		    }
		else                          //On est  la dernire range
		    {
			grille.setRowSelectionInterval(1, 1);   //On se place en premire range
			grille.setEditingRow(1);

			if (sc < RepresentationInterne.NB_COLONNES)  //On n'est pas  la dernire colonne -> se placer en ligne 1  la colonne suivante
			    {
				grille.editCellAt(1, sc + 1);
				grille.setColumnSelectionInterval(sc + 1, sc + 1);
				grille.setEditingColumn(sc + 1);
			    }
			else  //On est  la dernire colonne -> se placer en ligne 1, colonne 1
			    {
				grille.editCellAt(1,1);
				grille.setColumnSelectionInterval(1, 1);
				grille.setEditingColumn(1);
			    }
		    }

	    }
	else if ((key == KeyEvent.VK_UP) && (modifiers == KeyEvent.CTRL_MASK))   //La touche "ctrl+flche-haut" a t presse -> Monter d'une range si on n'est pas dj  la premire
	    {
		if (sr > 1)    
		    {
			grille.editCellAt(sr - 1, sc);
			grille.setRowSelectionInterval(sr - 1, sr - 1);
			grille.setEditingRow(sr - 1);
		    }
	    }
	else if ((key == KeyEvent.VK_DOWN) && (modifiers == KeyEvent.CTRL_MASK)) //La touche "ctrl+flche-bas" a t presse -> Descendre d'une range si on n'est pas dj  la dernire
	    {
		if (sr < RepresentationInterne.NB_LIGNES)
		    {
			grille.editCellAt(sr + 1, sc);
			grille.setRowSelectionInterval(sr + 1, sr + 1);
			grille.setEditingRow(sr + 1);
		    }
	    }
	else if ((key == KeyEvent.VK_LEFT) && (modifiers == KeyEvent.CTRL_MASK)) //La touche "ctrl+flche-gauche" a t presse -> Se dplacer d'une colonne vers la gauche si on n'est pas dj  la premire
	    {
		if (sc > 1)
		    {
			grille.editCellAt(sr, sc - 1);
			grille.setColumnSelectionInterval(sc - 1, sc - 1);
			grille.setEditingColumn(sc - 1);
		    }
	    }
	else if ((key == KeyEvent.VK_RIGHT) && (modifiers == KeyEvent.CTRL_MASK))  //La touche "ctrl+flche-droite" a t presse -> Se dplacer d'une colonne vers la droite si on n'est pas dj  la dernire
	    {
		if (sc < RepresentationInterne.NB_COLONNES)
		    {
			grille.editCellAt(sr, sc + 1);
			grille.setColumnSelectionInterval(sc + 1, sc + 1);
			grille.setEditingColumn(sc + 1);
		    }
	    }  

	
	else if ((key == KeyEvent.VK_DELETE) && (modifiers == KeyEvent.CTRL_MASK))  //On appuye sur la touche CTRL+DELETE -> effacer le contenu de la case slectionne dans le tableau
	    {
		grille.deleteSelectedArea();
	    }
	
    }

}
